#include <iostream>
#include <iterator>
#include "SortedVector.h"

using namespace std;

SortedVector::SortedVector()
{
    Node::reset();
}

SortedVector::~SortedVector()
{
    Node::reset();
}

void SortedVector::prepend(const long value)
{
    vector<Node>::insert(begin(), Node(value));
}

void SortedVector::append(const long value)
{
    push_back(Node(value));
}

void SortedVector::insert(const long value)
{
    vector<Node>::const_iterator it = begin();

    // Look for the appropriate position to insert.
    while ((it != end()) && (value > it->get_value())) it++;
    vector<Node>::insert(it, Node(value));
}

void SortedVector::remove(const int index)
{
    erase(begin() + index);
}

void SortedVector::reverse()
{
    vector<Node>::const_iterator it = begin() + 1;

    while (it != end())
    {
        vector<Node>::insert(begin(), *it);
        it++;
        erase(it);
    }
}

bool SortedVector::check() const
{
    if (size() == 0) return true;

    vector<Node>::const_iterator it = begin();
    vector<Node>::const_iterator prev = it;

    it++;

    // Ensure that each node is greater than or equal to the previous node.
    while ((it != end()) && (*it >= *prev))
    {
        prev = it;
        it++;
    }

    return it == end();  // Good if reached the end.
}

bool SortedVector::check_reversed() const
{
    if (size() == 0) return true;

    vector<Node>::const_iterator it = begin();
    vector<Node>::const_iterator prev = it;

    it++;

    // Ensure that each node is less than or equal to the previous node.
    while ((it != end()) && (*it <= *prev))
    {
        prev = it;
        it++;
    }

    return it == end();  // Good if reached the end.
}
